use crate::client::ONLINE_USERS;
use crate::config::config::REDIS_ADDR;
use lazy_static::lazy_static;
use redis::Client;
use redis::Commands;
use redis_pool::SingleRedisPool;
use std::collections::HashMap;
use urlencoding::decode;

lazy_static! {
    static ref REDIS_POOL: SingleRedisPool = {
        let client = Client::open(REDIS_ADDR).expect("Failed to connect to Redis");
        redis_pool::RedisPool::new(client, 16, Some(512))
    };
}

// 从 Redis 的 onlineUsers 集合中移除当前用户的信息
pub async fn remove_this_connection(from_id: &str) -> Result<(), redis::RedisError> {
    {
        ONLINE_USERS.remove(from_id);
        println!("成功从全局变量中移除用户id: {} 的信息", from_id);
    }

    let mut con = REDIS_POOL
        .get_connection()
        .expect("Failed to get Redis connection");

    if let Err(e) = con.hdel::<&str, &str, ()>("onlineUsers", from_id) {
        println!("从 Redis 中的 onlineUsers 哈希表删除用户信息时出错: {}", e);
        return Err(e);
    }
    Ok(())
}

pub async fn update_client_redis_data(
    from_id: &str,
    data_str: String,
) -> Result<(), redis::RedisError> {
    // 修改对应用户redis数据
    let mut con = REDIS_POOL
        .get_connection()
        .expect("Failed to get Redis connection");
    if let Err(e) = con.hset::<&str, &str, &str, ()>("onlineUsers", from_id, &data_str) {
        println!("修改 Redis 中的 onlineUsers 哈希表时出错: {}", e);
        return Err(e);
    }
    Ok(())
}

// 将当前用户的信息插入到 Redis 的 onlineUsers 集合中
pub async fn insert_this_connection(
    from_id: &str,
    params: &HashMap<String, String>,
) -> Result<(), redis::RedisError> {

    let device_id = params.get("deviceID").cloned().unwrap_or("".to_string());
    let from_name = params.get("fromName").cloned().unwrap_or("".to_string());
    let has_mike = if let Some(has_mike) = params.get("hasMike") {
        has_mike.to_string()
    } else {
        "0".to_string()
    };
    let has_camera = if let Some(has_camera) = params.get("hasCamera") {
        has_camera.to_string()
    } else {
        "0".to_string()
    };
    let user_call_group = if let Some(user_call_group) = params.get("userCallGroup") {
        user_call_group.to_string()
    } else {
        "0".to_string()
    };
    let from_name_decoded = match decode(from_name.as_str()) {
        Ok(decoded) => decoded.to_string(),
        Err(_) => from_name,
    };
    // 按照结构体OnlineUserMessage的格式构造用户信息字符串，用逗号拼接即可
    // callState,channelID,deviceID,fromID,hasCamera,hasMike,isHost,userCallGroup,fromName
    let user_info_str = format!(
        "{},{},{},{},{},{},0,{},{}",
        "idle", "", device_id, from_id, has_camera, has_mike, user_call_group, from_name_decoded
    );

    {
        // 获取写锁以进行写操作
        ONLINE_USERS.insert(from_id.to_string(), user_info_str.clone());
        println!("成功将用户id: {} 的信息插入到 ONLINE_USERS 中", from_id);
    }

    let mut con = REDIS_POOL
        .get_connection()
        .expect("Failed to get Redis connection");

    if let Err(e) = con.hset::<&str, &str, &str, ()>("onlineUsers", from_id, &user_info_str) {
        println!(
            "将用户信息添加到 Redis 中的 onlineUsers 哈希表时出错: {}",
            e
        );
        return Err(e);
    }
    Ok(())
}
